@@ -80,6 +80,7 @@ static void e_shortcut_bar_on_item_added	(EShortcutModel *model,
 						 gint		 item_num,
 						 gchar		*item_url,
 						 gchar		*item_name,
+						 GdkPixbuf      *item_image,
 						 EShortcutBar	*shortcut_bar);
 static void e_shortcut_bar_on_item_removed	(EShortcutModel *model,
 						 gint		 group_num,
@@ -90,6 +91,7 @@ static void e_shortcut_bar_on_item_updated	(EShortcutModel *model,
 						 gint		 item_num,
 						 gchar          *item_url,
 						 gchar          *item_name,
+						 GdkPixbuf      *item_image,
 						 EShortcutBar	*shortcut_bar);
 
 static gint e_shortcut_bar_add_group		(EShortcutBar	*shortcut_bar,
@@ -101,15 +103,17 @@ static gint e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 						 gint		 group_num,
 						 gint		 position,
 						 const gchar	*item_url,
-						 const gchar	*item_name);
+						 const gchar	*item_name,
+						 GdkPixbuf      *image);
 static void e_shortcut_bar_remove_item		(EShortcutBar	*shortcut_bar,
 						 gint		 group_num,
 						 gint		 item_num);
-static void e_shortcut_bar_update_item		(EShortcutBar *shortcut_bar,
-						 gint		 group_num,
-						 gint		 item_num,
-						 const gchar *item_url,
-						 const gchar *item_name);
+static void e_shortcut_bar_update_item		(EShortcutBar 	*shortcut_bar,
+						 gint         	 group_num,
+						 gint         	 item_num,
+						 const gchar  	*item_url,
+						 const gchar  	*item_name,
+						 GdkPixbuf    	*image);
 
 static void e_shortcut_bar_set_canvas_style	(EShortcutBar	*shortcut_bar,
 						 GtkWidget	*canvas);
@@ -143,9 +147,6 @@ static void e_shortcut_bar_on_drag_end		(GtkWidget      *widget,
 						 EShortcutBar   *shortcut_bar);
 static void e_shortcut_bar_stop_editing		(GtkWidget	*button,
 						 EShortcutBar	*shortcut_bar);
-static GdkPixbuf* e_shortcut_bar_get_image_from_url (EShortcutBar *shortcut_bar,
-						     const gchar *item_url);
-static GdkPixbuf* e_shortcut_bar_load_image	(const gchar	*filename);
 
 
 enum
@@ -286,6 +287,7 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 {
 	gint num_groups, group_num, num_items, item_num;
 	gchar *group_name, *item_url, *item_name;
+	GdkPixbuf *item_image;
 
 	/* Disconnect any existing model. */
 	e_shortcut_bar_disconnect_model (shortcut_bar);
@@ -326,12 +328,14 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 		for (item_num = 0; item_num < num_items; item_num++) {
 			e_shortcut_model_get_item_info (model, group_num,
 							item_num, &item_url,
-							&item_name);
+							&item_name, &item_image);
 			e_shortcut_bar_add_item (shortcut_bar, group_num,
 						 item_num, item_url,
-						 item_name);
+						 item_name, item_image);
 			g_free (item_url);
 			g_free (item_name);
+			if (item_image != NULL)
+				gdk_pixbuf_unref (item_image);
 		}
 	}
 }
@@ -385,10 +389,11 @@ e_shortcut_bar_on_item_added		(EShortcutModel *model,
 					 gint		 item_num,
 					 gchar		*item_url,
 					 gchar		*item_name,
+					 GdkPixbuf      *item_image,
 					 EShortcutBar	*shortcut_bar)
 {
 	e_shortcut_bar_add_item (shortcut_bar, group_num, item_num,
-				 item_url, item_name);
+				 item_url, item_name, item_image);
 }
 
 
@@ -407,10 +412,11 @@ e_shortcut_bar_on_item_updated (EShortcutModel *model,
 				gint            item_num,
 				gchar          *item_url,
 				gchar          *item_name,
+				GdkPixbuf      *image,
 				EShortcutBar   *shortcut_bar)
 {
 	e_shortcut_bar_update_item (shortcut_bar, group_num, item_num,
-				    item_url, item_name);
+				    item_url, item_name, image);
 }
 
 
@@ -523,10 +529,10 @@ e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 				 gint		 group_num,
 				 gint		 position,
 				 const gchar	*item_url,
-				 const gchar	*item_name)
+				 const gchar	*item_name,
+				 GdkPixbuf      *image)
 {
 	EShortcutBarGroup *group;
-	GdkPixbuf *image;
 	gint item_num;
 
 	g_return_val_if_fail (E_IS_SHORTCUT_BAR (shortcut_bar), -1);
@@ -535,13 +541,12 @@ e_shortcut_bar_add_item		(EShortcutBar	*shortcut_bar,
 	g_return_val_if_fail (item_url != NULL, -1);
 	g_return_val_if_fail (item_name != NULL, -1);
 
-	image = e_shortcut_bar_get_image_from_url (shortcut_bar, item_url);
 	group = &g_array_index (shortcut_bar->groups,
 				EShortcutBarGroup, group_num);
 
 	item_num = e_icon_bar_add_item (E_ICON_BAR (group->icon_bar),
 					image, item_name, position);
-	gdk_pixbuf_unref (image);
+
 	e_icon_bar_set_item_data_full (E_ICON_BAR (group->icon_bar), item_num,
 				       g_strdup (item_url), g_free);
 
@@ -571,10 +576,10 @@ e_shortcut_bar_update_item (EShortcutBar *shortcut_bar,
 			    gint          group_num,
 			    gint          item_num,
 			    const gchar  *item_url,
-			    const gchar  *item_name)
+			    const gchar  *item_name,
+			    GdkPixbuf    *image)
 {
 	EShortcutBarGroup *group;
-	GdkPixbuf *image;
 	EIconBar *icon_bar;
 	EIconBarItem *item;
 
@@ -584,7 +589,6 @@ e_shortcut_bar_update_item (EShortcutBar *shortcut_bar,
 	g_return_if_fail (item_url != NULL);
 	g_return_if_fail (item_name != NULL);
 
-	image = e_shortcut_bar_get_image_from_url (shortcut_bar, item_url);
 	group = &g_array_index (shortcut_bar->groups,
 				EShortcutBarGroup, group_num);
 
@@ -595,8 +599,6 @@ e_shortcut_bar_update_item (EShortcutBar *shortcut_bar,
 	e_icon_bar_set_item_image (icon_bar, item_num, image);
 	e_icon_bar_set_item_text (icon_bar, item_num, item_name);
 
-	gdk_pixbuf_unref (image);
-	
 	e_icon_bar_set_item_data_full (icon_bar, item_num,
 				       g_strdup (item_url), g_free);
 }
@@ -740,7 +742,7 @@ e_shortcut_bar_on_drag_data_received  (GtkWidget          *widget,
 	position = icon_bar->dragging_before_item_num;
 
 	if ((data->length >= 0) && (data->format == 8)
-	    &&  position != -1) {
+	    && (position != -1) && (info == TARGET_SHORTCUT)) {
 		item_name = data->data;
 		item_url = item_name + strlen (item_name) + 1;
 
@@ -827,61 +829,6 @@ e_shortcut_bar_stop_editing (GtkWidget *button,
 }
 
 
-/* Sets the callback which is called to return the icon to use for a particular
-   URL. */
-void
-e_shortcut_bar_set_icon_callback (EShortcutBar *shortcut_bar,
-				  EShortcutBarIconCallback cb,
-				  gpointer data)
-{
-	shortcut_bar->icon_callback = cb;
-	shortcut_bar->icon_callback_data = data;
-}
-
-
-static GdkPixbuf *
-e_shortcut_bar_get_image_from_url (EShortcutBar *shortcut_bar,
-				   const gchar *item_url)
-{
-	GdkPixbuf *icon = NULL;
-
-	if (shortcut_bar->icon_callback)
-		icon = (*shortcut_bar->icon_callback) (shortcut_bar,
-						       item_url,
-						       shortcut_bar->icon_callback_data);
-
-	if (!icon) {
-		if (!e_shortcut_bar_default_icon_loaded) {
-			e_shortcut_bar_default_icon_loaded = TRUE;
-			e_shortcut_bar_default_icon = e_shortcut_bar_load_image (e_shortcut_bar_default_icon_filename);
-		}
-		icon = e_shortcut_bar_default_icon;
-		/* ref the default icon each time we return it */
-		gdk_pixbuf_ref (icon);
-	}
-
-	return icon;
-}
-
-
-static GdkPixbuf *
-e_shortcut_bar_load_image (const gchar *filename)
-{
-	gchar *pathname;
-	GdkPixbuf *image = NULL;
-
-	pathname = gnome_pixmap_file (filename);
-	if (pathname)
-		image = gdk_pixbuf_new_from_file (pathname);
-	else
-		g_warning ("Couldn't find pixmap: %s", filename);
-
-	g_free (pathname);
-
-	return image;
-}
-
-
 /* Set whether items can be dragged, for drag-and-drop. */
 void
 e_shortcut_bar_set_enable_drags    (EShortcutBar	 *shortcut_bar,
